@@ -15,10 +15,12 @@
 # KIND, either express or implied.  See the License for the
 # specific language governing permissions and limitations
 # under the License.
-"""Hook for Web HDFS"""
+"""Hook for Web HDFS."""
+from __future__ import annotations
+
 import logging
 import socket
-from typing import Any, Optional
+from typing import Any
 
 import requests
 from hdfs import HdfsError, InsecureClient
@@ -39,7 +41,7 @@ if _kerberos_security_mode:
 
 
 class AirflowWebHDFSHookException(AirflowException):
-    """Exception specific for WebHDFS hook"""
+    """Exception specific for WebHDFS hook."""
 
 
 class WebHDFSHook(BaseHook):
@@ -50,7 +52,7 @@ class WebHDFSHook(BaseHook):
     :param proxy_user: The user used to authenticate.
     """
 
-    def __init__(self, webhdfs_conn_id: str = 'webhdfs_default', proxy_user: Optional[str] = None):
+    def __init__(self, webhdfs_conn_id: str = "webhdfs_default", proxy_user: str | None = None):
         super().__init__()
         self.webhdfs_conn_id = webhdfs_conn_id
         self.proxy_user = proxy_user
@@ -59,7 +61,6 @@ class WebHDFSHook(BaseHook):
         """
         Establishes a connection depending on the security mode set via config or environment variable.
         :return: a hdfscli InsecureClient or KerberosClient object.
-        :rtype: hdfs.InsecureClient or hdfs.ext.kerberos.KerberosClient
         """
         connection = self._find_valid_server()
         if connection is None:
@@ -68,44 +69,49 @@ class WebHDFSHook(BaseHook):
 
     def _find_valid_server(self) -> Any:
         connection = self.get_connection(self.webhdfs_conn_id)
-        namenodes = connection.host.split(',')
+        namenodes = connection.host.split(",")
         for namenode in namenodes:
             host_socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
             self.log.info("Trying to connect to %s:%s", namenode, connection.port)
             try:
                 conn_check = host_socket.connect_ex((namenode, connection.port))
                 if conn_check == 0:
-                    self.log.info('Trying namenode %s', namenode)
+                    self.log.info("Trying namenode %s", namenode)
                     client = self._get_client(
                         namenode,
                         connection.port,
                         connection.login,
+                        connection.get_password(),
                         connection.schema,
                         connection.extra_dejson,
                     )
-                    client.status('/')
-                    self.log.info('Using namenode %s for hook', namenode)
+                    client.status("/")
+                    self.log.info("Using namenode %s for hook", namenode)
                     host_socket.close()
                     return client
                 else:
                     self.log.warning("Could not connect to %s:%s", namenode, connection.port)
             except HdfsError as hdfs_error:
-                self.log.info('Read operation on namenode %s failed with error: %s', namenode, hdfs_error)
+                self.log.info("Read operation on namenode %s failed with error: %s", namenode, hdfs_error)
         return None
 
-    def _get_client(self, namenode: str, port: int, login: str, schema: str, extra_dejson: dict) -> Any:
-        connection_str = f'http://{namenode}'
+    def _get_client(
+        self, namenode: str, port: int, login: str, password: str | None, schema: str, extra_dejson: dict
+    ) -> Any:
+        connection_str = f"http://{namenode}"
         session = requests.Session()
+        if password is not None:
+            session.auth = (login, password)
 
-        if extra_dejson.get('use_ssl', 'False') == 'True' or extra_dejson.get('use_ssl', False):
-            connection_str = f'https://{namenode}'
-            session.verify = extra_dejson.get('verify', False)
+        if extra_dejson.get("use_ssl", "False") == "True" or extra_dejson.get("use_ssl", False):
+            connection_str = f"https://{namenode}"
+            session.verify = extra_dejson.get("verify", False)
 
         if port is not None:
-            connection_str += f':{port}'
+            connection_str += f":{port}"
 
         if schema is not None:
-            connection_str += f'/{schema}'
+            connection_str += f"/{schema}"
 
         if _kerberos_security_mode:
             return KerberosClient(connection_str, session=session)
@@ -118,7 +124,6 @@ class WebHDFSHook(BaseHook):
 
         :param hdfs_path: The path to check.
         :return: True if the path exists and False if not.
-        :rtype: bool
         """
         conn = self.get_conn()
 
@@ -148,3 +153,15 @@ class WebHDFSHook(BaseHook):
             hdfs_path=destination, local_path=source, overwrite=overwrite, n_threads=parallelism, **kwargs
         )
         self.log.debug("Uploaded file %s to %s", source, destination)
+
+    def read_file(self, filename: str) -> bytes:
+        """Read a file from HDFS.
+
+        :param filename: The path of the file to read.
+        :return: File content as a raw string
+        """
+        conn = self.get_conn()
+
+        with conn.read(filename) as reader:
+            content = reader.read()
+        return content
